﻿class GameState
{
	private static var _frameTime:Number;
	private static var _lastFrameTime:Number;
	private static var _player1:Player;
	private static var _enemies:Array;
	private static var _projectiles:Array;
	private static var _nextEnemyShipId:Number;
	private static var _nextProjectileId:Number;
	private static var _nextExplosionId:Number;
	private static var _shipDepth:Number;
	private static var _playerShipDepth:Number;
	private static var _projectileDepth:Number;
	private static var _explosionDepth:Number;
	private static var _activeScale:Number;
	private static var _majorLevel:Number;
	private static var _minorLevel:Number;
	private static var _nextLevelTime:Number;
	private static var _lastFlyByTime:Number;
	private static var _gameStartTime:Number;
	
	public static function initialize()
	{
		trace("game state initialized");
		
		SoundBoard.initialize();
		
		if (_projectiles != null)
			removeAllProjectiles();
		
		if (_enemies != null)		
			removeAllEnemyShips();
		
		_player1 = new Player();
		_enemies = new Array();
		_projectiles = new Array();
		_lastFrameTime = _frameTime = new Date().getTime();		
		_shipDepth = 999;
		_playerShipDepth = 500;
		_projectileDepth = 100000;
		_explosionDepth = 100000000;
		_nextEnemyShipId = 0;
		_nextProjectileId = 0;
		_nextExplosionId = 0;
		_activeScale = 0.20;
		_majorLevel = 1;
		_minorLevel = 0;
		_player1.setLives(3);
		_nextLevelTime = _lastFlyByTime = _frameTime;		
		_gameStartTime = _frameTime + 3500;
	}
	
	public static function gotoNextLevel()
	{
		if (_minorLevel < 10)
		{
			_minorLevel ++;
		}
		else
		{
			_majorLevel ++;
			_minorLevel = 1;
		}		
			
		_root.gameLevel.text = "Level : " + _majorLevel + "-" + _minorLevel;
		
		createLevel();
	}
	
	public static function getLastFlyByTime()
	{
		return _lastFlyByTime;
	}

	public static function setLastFlyByTime(value:Number)
	{
		_lastFlyByTime = value;
	}
	
	public static function getMajorLevel()
	{
		return _majorLevel;
	}
	
	public static function getMinorLevel()
	{
		return _minorLevel;
	}
	
	public static function isLevelComplete()
	{
		if (_enemies.length == 0)
			return true;
		else
			return false;
	}
	
	public static function processUserInput()	
	{
		if (Key.isDown(Key.RIGHT))
		{
			// move right									
			if (_player1.getShip()._x + _player1.getShip()._speed >= 
														  800 - (_player1.getShip()._width / 2))
				_player1.getShip()._x = 800 - (_player1.getShip()._width / 2);
			else
				_player1.getShip()._x += _player1.getShip()._speed;
		}								 
		if (Key.isDown(Key.LEFT))
		{
			// move left
			
			if (_player1.getShip()._x - _player1.getShip()._speed <= 
														  (_player1.getShip()._width / 2))
				_player1.getShip()._x = (_player1.getShip()._width / 2);
			else
				_player1.getShip()._x -= _player1.getShip()._speed;
		}								 		
		if (Key.isDown(Key.UP))
		{
			// move up
			
			if (_player1.getShip()._y - _player1.getShip()._speed <= 
														  (600 / 2) - (_player1.getShip()._height / 2))
				_player1.getShip()._y = (600 / 2) - (_player1.getShip()._height / 2);
			else
				_player1.getShip()._y -= _player1.getShip()._speed;
		}								 
		if (Key.isDown(Key.DOWN))
		{
			// move down
			
			if (_player1.getShip()._y + _player1.getShip()._speed >= 
														  600 - (_player1.getShip()._height / 2))
				_player1.getShip()._y = 600 - (_player1.getShip()._height / 2);
			else
				_player1.getShip()._y += _player1.getShip()._speed;
		}								 		
		if (Key.isDown(Key.SPACE))
		{
			// fire
			
			_player1.getShip().attack();		
		}								 
	}
	
	public static function processProjectiles()
	{
		for(var i:Number = 0; i < _projectiles.length; i ++)
			_projectiles[i].think();
	}
	
	public static function processEnemies()
	{		
		for(var i:Number = 0; i < _enemies.length; i ++)
			_enemies[i].think();
	}
	
	public static function think()
	{			
		// trace("frame begin");			
		_frameTime = new Date().getTime();		
		
		if (_frameTime < _gameStartTime)
			return;
			
		if (isLevelComplete() && _nextLevelTime == -1)
		{
			removeAllProjectiles();			

			if (_minorLevel == 9)
				_nextLevelTime = _frameTime + 7500;
			else
				_nextLevelTime = _frameTime + 5000;
		}
		else if (_nextLevelTime > -1 &&
				 _nextLevelTime < _frameTime)
		{
			gotoNextLevel();
			_nextLevelTime = -1;
		}
		
		processUserInput();
		
		processProjectiles();
		
		processEnemies();
		
		_lastFrameTime = _frameTime;
	}
	
	public static function getPlayer()
	{
		return _player1;
	}
	
	public static function removePlayerShip()
	{
		if (_player1.getShip() != null)
		{
			_player1.getShip().removeMovieClip();
			_player1.setShip(null);
		}
	}
	
	public static function removeEnemyShip(ship:Ship)
	{
		for(var i:Number = 0; i < _enemies.length; i ++)
		{
			if (_enemies[i] == ship)
			{
				_enemies[i].removeMovieClip();
				_enemies.splice(i, 1);

				if (_enemies.length == 0)
				{		
					_root.stopAllSounds();
					
					if (_minorLevel != 9 && _minorLevel != 10)
						SoundBoard.playSound(SoundBoard.SOUND_GRATZ_2);
					else if (_minorLevel == 10)
						SoundBoard.playSound(SoundBoard.SOUND_GRATZ_1);						
					else if (_minorLevel == 9)
						SoundBoard.playSound(SoundBoard.SOUND_BOSS_LEVEL);
				}				
				
				return;
			}
		}
	}
	
	public static function addEnemyShip(ship:Ship)
	{
		_enemies.push(ship);
	}
	
	public static function addProjectile(projectile:Projectile)
	{
		_projectiles.push(projectile);
	}	
	
	public static function removeProjectile(projectile:Projectile)
	{
		for(var i:Number = 0; i < _projectiles.length; i ++)
		{
			if (_projectiles[i] == projectile)
			{
				_projectiles[i].removeMovieClip();
				
				_projectiles.splice(i, 1);
				
				return;
			}
		}		
	}
	
	public static function removeAllProjectiles()
	{
		while(_projectiles.length > 0)
		{
			_projectiles[0].removeMovieClip();
				
			_projectiles.splice(0, 1);
		}		
	}	
	
	public static function removeAllEnemyShips()
	{
		while(_enemies.length > 0)
		{
			_enemies[0].removeMovieClip();
				
			_enemies.splice(0, 1);
		}		
	}
	
	public static function getNextEnemyShipId()
	{
		return _nextEnemyShipId ++;
	}
	
	public static function getNextProjectileId()
	{
		return _nextProjectileId ++;
	}
	
	public static function getNextExplosionId()
	{
		return _nextExplosionId ++;
	}
	
	public static function getPlayerShipIndex()
	{
		return 10;
	}
	
	public static function getFrameTime()
	{
		return _frameTime;
	}
	
	public static function getLastFrameTime()
	{
		return _lastFrameTime;
	}
	
	public static function getShipDepth()
	{
		return _shipDepth;
	}		
	
	public static function getPlayerShipDepth()
	{
		return _playerShipDepth;
	}	
	
	public static function getProjectileDepth()
	{
		return _projectileDepth;
	}	
	
	public static function getExplosionDepth()
	{
		return _explosionDepth;
	}
	
	public static function addPlayerShip(ship:PlayerShip)
	{
		_player1.setShip(ship);
	}
	
	public static function getActiveScale()
	{
		return _activeScale;
	}
	
	public static function checkEnemyCollision(other:MovieClip):Ship
	{
		for(var i:Number = 0; i < _enemies.length; i ++)
		{
			if (collisionTest(
							  _enemies[i],
							  other) == true)
			{
				return _enemies[i];
			}
		}
		
		return null;
	}	
	
	public static function checkPlayerCollision(other:MovieClip)
	{
		if (collisionTest(
						  _player1.getShip(),
						  other))
			return true;
		
		return false;
	}
	
	public static function collisionTest(
										 entity1:MovieClip,
										 entity2:MovieClip)
	{
		if (entity1 == null || entity2 == null)
			return false;
			
		var mins1x:Number, maxs1x:Number, mins2x:Number, maxs2x:Number;
		var mins1y:Number, maxs1y:Number, mins2y:Number, maxs2y:Number;
		
		// build boundaries for entity 1
		mins1x = entity1._x - (entity1._width / 2);
		maxs1x = entity1._x + (entity1._width / 2);		
		mins1y = entity1._y - (entity1._height / 2);
		maxs1y = entity1._y + (entity1._height / 2);				
		
		// build boundaries for entity 2
		mins2x = entity2._x - (entity2._width / 2);
		maxs2x = entity2._x + (entity2._width / 2);		
		mins2y = entity2._y - (entity2._height / 2);
		maxs2y = entity2._y + (entity2._height / 2);		
		
		if
			((mins1x <= mins2x && maxs1x >= mins2x &&
			mins1y <= mins2y && maxs1y >= mins2y) // top left
			||
			(mins1x <= maxs2x && maxs1x >= maxs2x &&
			mins1y <= mins2y && maxs1y >= mins2y) // top right
			||
			(mins1x <= mins2x && maxs1x >= mins2x &&
			mins1y <= maxs2y && maxs1y >= maxs2y) // bottom left
			||
			(mins1x <= maxs2x && maxs1x >= maxs2x &&
			mins1y <= maxs2y && maxs1y >= maxs2y)) // bottom right							
			{				
				return true;
			}
			else
			{
				return false;
			}
	}
	
	public static function createLevel()
	{
		if (_player1.getShip() == null)
		{
			trace("SHIP PLZ");
			createPlayerShip();
		}
			
		createEnemyShips();
	}
	
	public static function createPlayerShip()
	{		
		addPlayerShip(
								XWingPlayerShip.spawn(
													  400,
													  550,
													  -90,
													  0.20)); 
		_root.playerHealth.text = "Health : " + Math.round(_player1.getShip().getHealth()) + " Lives : " + GameState.getPlayer().getLives();;
	}
	
	public static function createEnemyShips()
	{		
		if (getMajorLevel() == 1)
		{
			//
			// Major level 1
			//
			
			if (getMinorLevel() == 1)
			{
				// Minor level 1
				
				trace("Creating level 1 - 1");
				
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M1Group1AttackPattern());
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M1Group2AttackPattern());
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M1Group3AttackPattern());
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M1Group4AttackPattern());
			}
			else if (getMinorLevel() == 2)
			{
				trace("Creating level 1 - 2");
				
				AttackPattern.spawnTieFigherCoupleFormation(
												  AttackPattern.createE1M2Group1AttackPattern());
				AttackPattern.spawnTieFigherCoupleFormation(
												  AttackPattern.createE1M2Group2AttackPattern());
			}
			else if (getMinorLevel() == 3)
			{
				trace("Creating level 1 - 3");
				
				AttackPattern.spawnTieFigherDiamondFormation(
												  AttackPattern.createE1M3Group1AttackPattern());
				AttackPattern.spawnTieFigherDiamondFormation(
												  AttackPattern.createE1M3Group2AttackPattern());
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M3Group3AttackPattern());
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M3Group4AttackPattern());
			}
			else if (getMinorLevel() == 4)
			{
				trace("Creating level 1 - 4");
				
				AttackPattern.spawnTieFigherBomberHalfCircleFormation(
												  AttackPattern.createE1M4Group1AttackPattern());
				AttackPattern.spawnTieFigherCoupleFormation(
												  AttackPattern.createE1M4Group2AttackPattern());
				AttackPattern.spawnTieFigherCoupleFormation(
												  AttackPattern.createE1M4Group3AttackPattern());
			}
			else if (getMinorLevel() == 5)
			{
				trace("Creating level 1 - 5");
				
				AttackPattern.spawnTieFigherBomberFullScreenFormation(
												  AttackPattern.createE1M5Group1AttackPattern());
			}
			else if (getMinorLevel() == 6)
			{
				trace("Creating level 1 - 6");
				
				AttackPattern.spawnTieFigherTripleFormation(
												  AttackPattern.createE1M6Group1AttackPattern());
				AttackPattern.spawnTieFigherBomberCircleFormation(
												  AttackPattern.createE1M6Group2AttackPattern());
				AttackPattern.spawnTieFigherBomberCircleFormation(
												  AttackPattern.createE1M6Group3AttackPattern());				
			}
			else if (getMinorLevel() == 7)
			{
				trace("Creating level 1 - 7");
				
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M7Group1AttackPattern());
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M7Group2AttackPattern());
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M7Group3AttackPattern());
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M7Group4AttackPattern());
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M7Group5AttackPattern());				
				
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M7Group6AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M7Group7AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M7Group8AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M7Group9AttackPattern());				
			}			
			else if (getMinorLevel() == 8)
			{
				trace("Creating level 1 - 8");
				
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M8Group1AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M8Group2AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M8Group3AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M8Group4AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M8Group5AttackPattern());				
			}
			else if (getMinorLevel() == 9)
			{
				trace("Creating level 1 - 9");
				
				AttackPattern.spawnTieFigherCoupleFormation(
												  AttackPattern.createE1M9Group1AttackPattern());
				AttackPattern.spawnTieFigherCoupleFormation(
												  AttackPattern.createE1M9Group2AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M9Group3AttackPattern());
			}
			else if (getMinorLevel() == 10)
			{
				trace("Creating level 1 - 10");
				
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M10Group1AttackPattern());
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M10Group2AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M10Group3AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M10Group4AttackPattern());				
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M10Group5AttackPattern());
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M10Group6AttackPattern());
				AttackPattern.spawnStarDestroyerSingleFormation(
												  AttackPattern.createE1M10Group7AttackPattern());				
				}			
		}
		else if (getMajorLevel() == 2)
		{
			//
			// Major level 2
			//
			
			if (getMinorLevel() == 1)
			{
				// Minor level 1
				
				trace("Creating level 2 - 1");
				
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M1Group1AttackPattern());
				AttackPattern.spawnTieInterceptorSingleFormation(
												  AttackPattern.createE1M1Group2AttackPattern());
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M1Group3AttackPattern());
				AttackPattern.spawnTieInterceptorSingleFormation(
												  AttackPattern.createE1M1Group4AttackPattern());
			}
			else if (getMinorLevel() == 2)
			{
				trace("Creating level 2 - 2");
				
				AttackPattern.spawnTieFigherCoupleFormation(
												  AttackPattern.createE1M2Group1AttackPattern());
				AttackPattern.spawnTieFigherCoupleFormation(
												  AttackPattern.createE1M2Group2AttackPattern());
			}
			else if (getMinorLevel() == 3)
			{
				trace("Creating level 2 - 3");
				
				AttackPattern.spawnTieFigherDiamondFormation(
												  AttackPattern.createE1M3Group1AttackPattern());
				AttackPattern.spawnTieInterceptorDiamondFormation(
												  AttackPattern.createE1M3Group2AttackPattern());
				AttackPattern.spawnTieInterceptorSingleFormation(
												  AttackPattern.createE1M3Group3AttackPattern());
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M3Group4AttackPattern());
			}
			else if (getMinorLevel() == 4)
			{
				trace("Creating level 2 - 4");
				
				AttackPattern.spawnTieFigherBomberHalfCircleFormation(
												  AttackPattern.createE1M4Group1AttackPattern());
				AttackPattern.spawnTieFigherCoupleFormation(
												  AttackPattern.createE1M4Group2AttackPattern());
				AttackPattern.spawnTieInterceptorCoupleFormation(
												  AttackPattern.createE1M4Group3AttackPattern());
			}
			else if (getMinorLevel() == 5)
			{
				trace("Creating level 2 - 5");
				
				AttackPattern.spawnTieFigherBomberInterceptorFullScreenFormation(
												  AttackPattern.createE1M5Group1AttackPattern());
			}
			else if (getMinorLevel() == 6)
			{
				trace("Creating level 2 - 6");
				
				AttackPattern.spawnTieInterceptorDiamondFormation(
												  AttackPattern.createE1M6Group1AttackPattern());
				AttackPattern.spawnTieFigherBomberCircleFormation(
												  AttackPattern.createE1M6Group2AttackPattern());
				AttackPattern.spawnTieFigherBomberCircleFormation(
												  AttackPattern.createE1M6Group3AttackPattern());				
			}
			else if (getMinorLevel() == 7)
			{
				trace("Creating level 2 - 7");
				
				AttackPattern.spawnTieInterceptorSingleFormation(
												  AttackPattern.createE1M7Group1AttackPattern());
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M7Group2AttackPattern());
				AttackPattern.spawnTieInterceptorSingleFormation(
												  AttackPattern.createE1M7Group3AttackPattern());
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M7Group4AttackPattern());
				AttackPattern.spawnTieInterceptorSingleFormation(
												  AttackPattern.createE1M7Group5AttackPattern());				
				
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M7Group6AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M7Group7AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M7Group8AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M7Group9AttackPattern());				
			}			
			else if (getMinorLevel() == 8)
			{
				trace("Creating level 2 - 8");
				
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M8Group1AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M8Group2AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M8Group3AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M8Group4AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M8Group5AttackPattern());				
			}
			else if (getMinorLevel() == 9)
			{
				trace("Creating level 2 - 9");
				
				AttackPattern.spawnTieFigherCoupleFormation(
												  AttackPattern.createE1M9Group1AttackPattern());
				AttackPattern.spawnTieInterceptorCoupleFormation(
												  AttackPattern.createE1M9Group2AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M9Group3AttackPattern());
			}
			else if (getMinorLevel() == 10)
			{
				trace("Creating level 2 - 10");
				
				AttackPattern.spawnTieInterceptorSingleFormation(
												  AttackPattern.createE1M10Group1AttackPattern());
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M10Group2AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M10Group3AttackPattern());
				AttackPattern.spawnTieBomberSingleFormation(
												  AttackPattern.createE1M10Group4AttackPattern());				
				AttackPattern.spawnTieFigherSingleFormation(
												  AttackPattern.createE1M10Group5AttackPattern());
				AttackPattern.spawnTieInterceptorSingleFormation(
												  AttackPattern.createE1M10Group6AttackPattern());
				AttackPattern.spawnStarDestroyerSingleFormation(
												  AttackPattern.createE1M10Group7AttackPattern());				
			}			
		}
		else if (getMajorLevel() == 3)	
		{
			_root.gotoAndStop("win");
		}
	}
}