﻿class Projectile extends MovieClip
{
	private var _fireSound:Sound;
	private var _flySound:Sound;
	private var _hitSound:Sound;
	private var _expireSound:Sound;
	
	
	private var _damage:Number;
	private var _speed:Number;
	private var _ownerShip:Ship;
	private var _born:Number;
	private var _maxAge:Number;
	
	// projectile movement logic
	public function move()
	{		
		// nothing special here, basic directional movement
		_y += Math.sin((_rotation) * (Math.PI/180)) * _speed;
		_x += Math.cos((_rotation) * (Math.PI/180)) * _speed;
	}
	
	public function destroy()
	{
		if (_expireSound != null)
			_expireSound.start();		
			
		GameState.removeProjectile(this);		
	}
	
	public function explode()
	{
		if (_hitSound != null)
			_hitSound.start();		
		
		GameState.removeProjectile(this);		
	}
	
	public function findCollision():Ship
	{
		if (_ownerShip == null)
			return null;
			
		if (!_ownerShip.isPlayer())
		{
			if (GameState.checkPlayerCollision(this))
				return GameState.getPlayer().getShip();
			else
				return null;
		}
		else
		{
			return GameState.checkEnemyCollision(this);
		}
	}
	
	public function think()
	{
		if (GameState.getFrameTime() + _born > _maxAge)
		{
			destroy();
			return;
		}
						
		if (_x < -1000 || _x > 2000 ||
			_y < -1000 || _y > 2000)
		{
			explode();
			return;
		}
		
		move();
		
		var collisionEntity:Ship = findCollision();		
		
		if (collisionEntity != null)
		{						
			collisionEntity.damage(_damage);
			
			destroy();
		}				
	}

	// spawns
	static function spawnConcussion(
						  	   owner:Ship,
							   x:Number,
							   y:Number,
							   rotation:Number,
							   scale:Number)
	{
		//trace("Creating concussion projectile");
		
		var index:Number = GameState.getNextProjectileId();
		var identifier:String = "ConcussionProjectile" + index;		
		
		var projectile:MovieClip = _root.attachMovie(
			"Concussion",
			identifier,
			GameState.getProjectileDepth() + index);						  
		
		// size and position the shit!
		projectile._ownerShip = owner;
		projectile._x = x;
		projectile._y = y;
		projectile._height *= scale;
		projectile._width *= scale;
		projectile._rotation = rotation;
		projectile._speed = 2500;
		projectile._maxAge = 2000;
		projectile._fireSound = SoundBoard.SOUND_BLASTER_XWING;
		projectile._expireSound = SoundBoard.SOUND_BLASTER_HIT;
		projectile._damage = 5;
		
		projectile._fireSound.start();
		
		return projectile;
	}	
	
	static function spawnIonCannon(
						  	   owner:Ship,
							   x:Number,
							   y:Number,
							   rotation:Number,
							   scale:Number)
	{
		//trace("Creating ion cannon projectile");
		
		var index:Number = GameState.getNextProjectileId();
		var identifier:String = "IonCannonProjectile" + index;		
		
		var projectile:MovieClip = _root.attachMovie(
			"IonCannon",
			identifier,
			GameState.getProjectileDepth() + index);						  
		
		// size and position the shit!
		projectile._ownerShip = owner;
		projectile._x = x;
		projectile._y = y;
		projectile._height *= scale;
		projectile._width *= scale;
		projectile._rotation = rotation;
		projectile._speed = 20;
		projectile._maxAge = 2000;
		projectile._fireSound = SoundBoard.SOUND_BLASTER_TIE_FIGHTER;
		projectile._expireSound = SoundBoard.SOUND_BLASTER_HIT;
		projectile._damage = 5;
		
		projectile._fireSound.start();
		
		return projectile;
	}	
	
	static function spawnPhotonTorpedo(
						  	   owner:Ship,
							   x:Number,
							   y:Number,
							   rotation:Number,
							   scale:Number)
	{
		//trace("Creating proton projectile");
		
		var index:Number = GameState.getNextProjectileId();
		var identifier:String = "ProtonProjectile" + index;		
		
		var projectile:MovieClip = _root.attachMovie(
			"Proton",
			identifier,
			GameState.getProjectileDepth() + index);						  
		
		// size and position the shit!
		projectile._ownerShip = owner;
		projectile._x = x;
		projectile._y = y;
		projectile._height *= scale;
		projectile._width *= scale;
		projectile._rotation = rotation;
		projectile._speed = 11;
		projectile._maxAge = 2000;
		projectile._fireSound = SoundBoard.SOUND_BLASTER_PHOTON;
		projectile._expireSound = SoundBoard.SOUND_BLASTER_HIT;
		projectile._damage = 200;
		
		projectile._fireSound.start();
		
		return projectile;
	}	
	
	static function spawnRebelBlaster(
						  	   owner:Ship,
							   x:Number,
							   y:Number,
							   rotation:Number,
							   scale:Number)
	{
		//trace("Creating rebel blaster projectile");
		
		var index:Number = GameState.getNextProjectileId();
		var identifier:String = "RebelBlasterProjectile" + index;		
		
		var projectile:MovieClip = _root.attachMovie(
			"RebelBlaster",
			identifier,
			GameState.getProjectileDepth() + index);						  
		
		// size and position the shit!
		projectile._ownerShip = owner;
		projectile._x = x;
		projectile._y = y;
		projectile._height *= scale;
		projectile._width *= scale;
		projectile._rotation = rotation;
		projectile._speed = 25;
		projectile._maxAge = 2000;
		projectile._fireSound = SoundBoard.SOUND_BLASTER_XWING;
		projectile._expireSound = SoundBoard.SOUND_BLASTER_HIT;
		projectile._damage = 10;
		
		projectile._fireSound.start();
		
		return projectile;
	}	
	
	static function spawnTurboLaser(
						  	   owner:Ship,
							   x:Number,
							   y:Number,
							   rotation:Number,
							   scale:Number)
	{
		//trace("Creating turbo laser projectile");
		
		var index:Number = GameState.getNextProjectileId();
		var identifier:String = "TurboLaserProjectile" + index;		
		
		var projectile:MovieClip = _root.attachMovie(
			"TurboLaser",
			identifier,
			GameState.getProjectileDepth() + index);						  
		
		// size and position the shit!
		projectile._ownerShip = owner;
		projectile._x = x;
		projectile._y = y;
		projectile._height *= scale;
		projectile._width *= scale;
		projectile._rotation = rotation;
		projectile._maxAge = 2000;
		projectile._fireSound = SoundBoard.SOUND_BLASTER_TIE_INTERCEPTOR;
		projectile._expireSound = SoundBoard.SOUND_BLASTER_HIT;
		projectile._speed = 20;
		projectile._damage = 10;
		
		if (!owner.isPlayer())
		{
			projectile._speed = owner.getProjectileSpeed() * owner.getAttackPattern().getProjectileSpeedMultiplier();		
			projectile._damage = owner.getAttackDamage() * owner.getAttackPattern().getDamageMultiplier();
		}
		else
		{
			projectile._speed = owner.getProjectileSpeed();		
			projectile._damage = owner.getAttackDamage();
		}
		
		projectile._fireSound.start();
		
		return projectile;
	}	
}