﻿class Ship extends MovieClip
{		
	private var _shipId:String;					// id of the ship
	private var _speed:Number = 5;				// current speed
	private var _lastFireTime:Number = 0;		// last time ship fired
	private var _fireDelay:Number = 500;		// delay between weapon firing
	private var _health:Number = 50;			// ship health
	private var _attackDamage:Number = 5;		// attack damage
	private var _projectileSpeed:Number = 10;	// attack damage	
	private var _isPlayer:Boolean = false;		// is this ship a player ship

	private var _explosionMovieClipName:String = "LargeExplosion"; // Movie clip to play for explosion
	
	// ship movement logic
	public function move()
	{
		// nothing special here, basic directional movement						
		_y += Math.sin((_rotation) * (Math.PI/180)) * _speed;
		_x += Math.cos((_rotation) * (Math.PI/180)) * _speed;
	}
	
	public function getHealth()
	{
		return _health;
	}
	
	public function getAttackPattern()
	{
		return null;
	}
	
	public function getAttackDamage()
	{
		return _attackDamage;
	}
	
	public function getProjectileSpeed()
	{
		return _projectileSpeed;
	}
	
	public function canAttack()
	{		
		return (_lastFireTime + _fireDelay) < GameState.getFrameTime();
	}
	
	public function attack()
	{					
		_lastFireTime = GameState.getFrameTime();
	}

	public function isPlayer()
	{
		return _isPlayer;
	}
	
	public function destroy()
	{
		if (_explosionMovieClipName != null && _explosionMovieClipName != "")
		{
			var explosionIndex:Number = GameState.getNextExplosionId();
			
			var explosion:MovieClip = _root.attachMovie(
						  _explosionMovieClipName,
						  "Explosion" + explosionIndex,
						  GameState.getExplosionDepth() + explosionIndex);						
			
			explosion._x = _x;
			explosion._y = _y;
		}
		
		if (!_isPlayer)
		{
			GameState.removeEnemyShip(this);		
		}
		else
		{
			SoundBoard.playSound(SoundBoard.SOUND_TAUNT_1);
			
			GameState.removePlayerShip();
			
			GameState.getPlayer().decrementLives();
			
			if (GameState.getPlayer().getLives() <= 0)
			{			
				_root.gotoAndStop("gameover");
			}
			else
			{
				GameState.createPlayerShip();
			}
		}
	}
	
	public function damage(amount:Number)
	{
		if (_isPlayer)		
			_root.playerHealth.text = "Health : " + (Math.round(_health - amount)) + " Lives : " + GameState.getPlayer().getLives();
		
		if (_health <= amount)
		{
			destroy();
			return;
		}		
		
		_health -= amount;
	}
}